# subs2vec
[Van Paridon & Thompson (2019)](https://doi.org/10.31234/osf.io/fcrmy) introduces pretrained embeddings and precomputed word/bigram/trigram frequencies in 55 languages. The files can be downloaded from the links in __[this table](#downloading-datasets)__. Word vectors trained on subtitles are available, as well as vectors trained on Wikipedia, and a combination of subtitles and Wikipedia (for best predictive performance).

This repository contains the subs2vec module, a number of Python 3.7 scripts and command line tools to evaluate a set of word vectors on semantic similarity, semantic and syntactic analogy, and lexical norm prediction tasks. In addition, the `subs2vec.py` script will take an OpenSubtitles archive or Wikipedia and go through all the steps to train a fastText model and produce word vectors as used in the paper associated with this repository.  

Psycholinguists may be especially interested `norms` script, which evaluates the lexical norm prediction performance of a set of word vectors, but can also be used to predict lexical norms for un-normed words. For a more detailed explanation see the __[how to use -> extending lexical norms](#extending-lexical-norms)__ section.  

The scripts in this repository require [Python 3.7](https://www.python.org/downloads/) and some additional libraries that are easily installed through pip. (If you want to use the `subs2vec.py` script to train your own word embeddings, you will also need compiled fastText and word2vec binaries.)  

If you use any of the subs2vec code and/or pretrained models, please cite the preprint ([Van Paridon & Thompson, 2019](https://doi.org/10.31234/osf.io/fcrmy)).  

## How to use
subs2vec is available through pip, installing is as easy as running:  
`python3 -m pip install subs2vec`  
Any missing dependencies should be installed automatically.  

Each submodules of subs2vec can then be run as a command line tool using the -m flag:  
`python3 -m subs2vec.submodule_name`  

### Evaluating word embeddings
To evaluate word embeddings on analogies, semantic similarity, or lexical norm prediction as in Van Paridon & Thompson (2019), use:  
`python3 -m subs2vec.analogies fr french_word_vectors.vec`  
`python3 -m subs2vec.similarities fr french_word_vectors.vec`  
`python3 -m subs2vec.norms fr french_word_vectors.vec`  
subs2vec uses the two-letter ISO language codes, so French in the example is `fr`, English would be `en`, German would be `de`, etc.

All datasets used for evaluation, including the lexical norms, are stored in `subs2vec/evaluation/datasets/`.  
Results from [Van Paridon & Thompson (2019)](https://doi.org/10.31234/osf.io/fcrmy) are in `subs2vec/evaluation/article_results/`.

### Extending lexical norms
To extend lexical norms (either norms you have collected yourself, or norms provided in this repository) use:  
`python3 -m subs2vec.norms fr french_word_vectors.vec --extend_norms=french_norms_file.txt`  

The norms file should be a tab-separated text file, with the first line containing column names and the column containing the words should be called `word`. Unobserved cells should be left empty. If you are unsure how to generate this file, you can create your list in Excel and then use `Save as... tab-delimited text`.  
For an overview of norms that come included in the repo (and their authors), see [this list](https://github.com/jvparidon/subs2vec/blob/master/subs2vec/datasets/norms_table.tsv). For the norms datasets themselves, look inside [this directory](https://github.com/jvparidon/subs2vec/tree/master/subs2vec/datasets/norms).

### Extracting word frequencies
The subtitle corpus used to train subs2vec was also used to compile the word frequencies in [SUBTLEX](https://www.frontiersin.org/articles/10.3389/fpsyg.2011.00027/full). That same corpus can of course be used to compile bigram and trigram frequencies as well.  
To extract word, bigram, or trigram frequencies from a text file yourself, `fr.txt` for instance, use:  
`python3 -m subs2vec.frequencies fr.txt`  

In general, however, we recommend downloading the precompiled frequencies files from [language archive] and looking frequencies up in those.  
When looking up frequencies for specific words, bigrams, or trigrams, you may find that you cannot open the frequencies files (they can be very large). To retrieve items of interest use:   
`python3 -m subs2vec.lookup frequencies_file.tsv list_of_items.txt`  
Your list of items should be a simple text file, with each item you want to look up on its own line.  
This lookup scripts works for looking up frequencies, but it finds lines in any plain text file, so it works for looking up word vectors in .vec files as well.

### Removing duplicate lines
subs2vec comes with a module that removes duplicate lines from text files. We used it to remove duplicate lines from training corpora, but it works for any text file.  
To remove duplicates from `fr.txt` for example, use:  
`python3 -m subs2vec.deduplicate fr.txt`

### Training models
If you want to reproduce models as used in [Van Paridon & Thompson (2019)](https://doi.org/10.31234/osf.io/fcrmy), you can use the `train_model` module.  
For instance, the steps to create a subtitle corpus are:
1. Download a corpus:  
`python3 -m subs2vec.download fr subs`  
2. Clean the corpus:  
`python3 -m subs2vec.clean_subs fr --strip --join`  
3. Deduplicate the lines in the corpus:  
`python3 -m subs2vec.deduplicate fr.txt`  
4. Train a fastText model on the subtitle corpus:  
`python3 -m subs2vec.train_model fr subs dedup.fr.txt`  
This last step requires the binaries for [fastText](https://github.com/facebookresearch/fastText) and [word2phrase (part of word2vec)](https://github.com/tmikolov/word2vec) to be downloaded, built, and discoverable on your system (i.e., on your PATH).

For more detailed training options:  
`python3 -m subs2vec.train_model --help`

## API
For more detailed documentation of the package modules and API, see [subs2vec.readthedocs.io](https://subs2vec.readthedocs.io)

## Downloading datasets
This table contains links to the top 1 million word vectors in each language, as well all vectors, model binaries, and the word, bigram, and trigram frequencies in the subtitle and Wikipedia corpora. If you use these pretrained vectors/models, please cite the preprint ([Van Paridon & Thompson, 2019](https://doi.org/10.31234/osf.io/fcrmy)).  

| language | lang | corpus | vectors | corpus word count | ngram frequencies |
|---|---|---|---|---:|---|
| Afrikaans | af | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_59a61205_dfc3_4eec_9fd9_311b3f65c9a5/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_486caf84_5d28_439c_a09f_6b90b7fe79dd/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_8d87878f_ce0b_4221_81b3_5fff04eec335/datastream/OBJ/download) |    323K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_a1a82d13_be9d_4024_bbfe_90de100fcd3c/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_1f671cfa_131d_4fa8_bfe2_021a59952256/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_e4a8e2f0_2cfa_4d5a_8d22_864ef02d7f26/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors]()<br>[all vectors]()<br>[model binary]() |     17M | [word counts]()<br>[bigram counts]()<br>[trigram counts]() |
| | | Wikipedia + OpenSubtitles | [top 1M vectors]()<br>[all vectors]()<br>[model binary]() | | |
| Arabic | ar | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_4d9c22fc_fa2e_4172_868b_2ee42101cc46/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_dbdda088_0c75_43be_93e2_96d91851375f/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_c2402eb5_a5c5_4c3d_b812_cd067155ed9c/datastream/OBJ/download) |    188M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_2f747bf3_5974_493e_b541_b608660843e5/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_20bd0262_8f9b_451d_a2ef_2aa3f67dc919/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_1f15561c_18fb_4386_a027_cec5c9214223/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_d76d8c0e_6ea9_43b8_862c_41fbf0fda997/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_921f5077_087f_4aa6_9062_9e80f99c2749/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_6fe8e7a5_f69a_4b7d_bc8b_1c0c1c115e44/datastream/OBJ/download) |     119M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_277a81e3_d6e5_4329_9d14_443362dcba3c/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_599cc779_486d_4d54_bf69_472586c6c478/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_49998cd2_f46a_4aa9_8f04_5e977596ffad/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_3f173eeb_a56e_451a_9713_8311a7e777c1/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_e2c926d4_ccb3_4fdb_907d_fe44b878b123/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_6349f448_2506_417b_bdfe_ed6cbf240f5b/datastream/OBJ/download) | | |
| Bulgarian | bg | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_2c35acb4_9da0_474a_b75b_d827a6514e54/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_a6f3034d_d5eb_4b83_8ede_3f55ad888488/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_60303dde_aa64_44ce_b2fc_729b63f24fba/datastream/OBJ/download) |    246M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_53ad8a0a_4bd9_49ea_bfe8_fecd95664912/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_89115179_7319_40ff_9a90_49f26ec550f4/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_e9c22c29_8ebd_47a9_8dd8_e7af8000189a/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_af30b5a1_c403_43d7_b3e1_9e771910a4bc/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_269b9008_46ae_45aa_a854_b6ae878ae6f1/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_0b087c91_650a_4abc_a828_5b1c945ee2a6/datastream/OBJ/download) |     53M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_a2cebd0a_2c26_4e70_a045_d3bd9b37c842/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_a0be309f_1fb6_42cb_b70a_b253bc728c00/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_024f4d5f_5fac_4423_b67c_4d6b013be0f2/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_9370ac19_48b2_4a59_b4f3_c263db800b33/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_ac329e8f_3eb3_4e4e_925e_addf4a72dca9/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_84c9b31a_ec25_4668_b949_6ed8d0e1949c/datastream/OBJ/download) | | |
| Bengali | bn | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_8597ff89_4f75_4e75_a45a_695063ae90b3/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_6c8aa06f_d68a_450d_b88a_11bcbfe1daea/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_4151ed7c_2b9d_4b36_b344_2111ebe8779f/datastream/OBJ/download) |    2227K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_7b1d3bf6_0191_4d8b_8df3_05e97b33e45a/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_5e735b0f_7d2e_460a_9267_689bc2c1c4bf/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_34ebb7f3_4a69_4458_81f7_b8d9a91a7db9/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_fdb33a71_86a4_49e6_b44f_da968e85f633/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_5691e89f_e15f_4f15_bf40_f7f292dd5ff6/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_090c2958_0969_4217_bc29_2543aa099b8b/datastream/OBJ/download) |     18M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_e68ef982_5116_4489_89bd_f84d5ff0d4f5/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_e62b0537_97cc_4fd6_bb84_36d7ff5f6db7/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_2873b5b0_ef88_4e56_95da_7d8167dd6d35/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_9fa22138_15e5_4276_9c17_7862b1eb78d3/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_c8bebaf8_f5aa_4e1c_b817_c6ef39fbf6fc/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_7bab3265_f053_423c_82c0_a7a8b4b2578e/datastream/OBJ/download) | | |
| Breton | br | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_5c33cc0a_e668_467f_97cc_2e3efafe5d1b/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_c7abe795_069e_46e8_a6b4_c14ed70f9f72/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_26f017fd_92c8_49d4_bbf1_1c210b9528c1/datastream/OBJ/download) |    110K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_315f5dba_662b_461b_b911_e46ca62553ba/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_64def660_f4ba_41b0_a6f1_e9286eb921a1/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_3f72b094_0750_455c_b110_171e7439c503/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_166942c5_b0f7_4698_b921_cd57259d68ab/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_73a6422a_efec_485a_97c7_7984818d23f4/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_d47fbee1_e0fe_4bf6_8001_3f984db1aad5/datastream/OBJ/download) |     7644K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_12db32c9_4f87_4b99_b8ec_ac0b92078dd2/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_416b702b_c05d_4ae9_9920_c34708fe5ad6/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_71c8d12a_70a0_48b4_8311_91c97e2eedca/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_5dc968c8_e34a_4cfb_8c23_d66906496715/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_1910a62b_8caf_4318_af6e_63ff4208379b/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_87332304_6d12_4af1_8f17_fdb0977456ae/datastream/OBJ/download) | | |
| Bosnian | bs | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_53a8bb6c_b595_4d41_95e7_555f8a6df93a/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_becf935e_7f94_4614_ac8b_80f569fe00a2/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_5c4bb28f_c6c3_4bf5_b0ad_795cace966fe/datastream/OBJ/download) |    91M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_e09888f2_d893_4668_98e8_3d667e6feee3/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_d8719ddb_afed_453e_a2e0_f7315ee362f7/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_f23f21e7_ba44_451a_8de3_7ce9044af418/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_fd9f9c36_f8cf_489c_85eb_06412da42214/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_9db8971a_ed7f_4003_8823_d95637a25cb2/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_c8724f99_41e9_419b_9986_6a626a71c833/datastream/OBJ/download) |     13M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_1eb74604_429d_4a19_8562_65187d76f3e5/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_4fa4801d_3c3f_450f_8242_cd02bf07fa2c/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_87916c34_9124_4696_b986_79df88a67d14/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_6396ea9f_49d7_4cd4_8797_b6c0489c55d3/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_b5b66654_2b32_4814_8b50_4c41a6abd28c/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_75b04482_68d0_417d_a5c7_698fd4ecb8be/datastream/OBJ/download) | | |
| Catalan | ca | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_96562977_b1db_486d_96d4_bd3c5a6bfdf9/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_a448480b_c1bc_4ceb_b717_11bb90a36df8/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_01966136_9fdb_43e3_9b63_4dfcbc1181f1/datastream/OBJ/download) |    3098K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_60997108_f112_412f_ad9b_39f8b66950ff/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_99482332_a447_4e85_a221_016266afa253/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_15f6f181_ab0e_44ac_b24b_78a66821091b/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_7fe44c36_deb3_4417_a2f2_a23b3a513d2b/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_0ea66ecd_46a6_45a1_ae7f_028eb192d88c/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_7a9c2f42_8e69_4885_8251_4a4e9b96c58f/datastream/OBJ/download) |     175M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_7e40aa5b_eb93_4698_9c06_861ed1059ea9/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_09b16187_6179_4dac_a0ae_53d8321a6169/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_46dedd93_6922_42aa_8492_295d48fcc018/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_07067e9a_b017_4a53_9b64_916ad8b80a55/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_1ff2cd5b_070b_4341_bc3e_5d3a208a05ea/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_cd9b7dc6_3b0e_4d5e_82a4_2a6a4e49f77a/datastream/OBJ/download) | | |
| Czech | cs | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_d61ab6d0_ec89_486a_a57e_08dcf26bae65/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_3683be29_bd28_4c16_b330_a626b7649e88/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_2214b667_b80f_4ffd_89b6_d7c757e3215f/datastream/OBJ/download) |    249M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_40a0c81b_3da5_44d8_8bcf_0b18d6ebf3c8/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_dca8b302_7e14_40a8_a12f_7a549aaae454/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_befd2bee_b401_48f5_9f63_610091ba3b69/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_24f417b0_8e91_442d_a4dd_9b5b1792367d/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_3062b2e7_17f7_4f1d_b488_f798ac3e9283/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_ad6cea21_034e_492e_9ae6_5c8348b02b52/datastream/OBJ/download) |     100M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_0b007cb2_03bc_4115_a6dd_15ae9ba4e607/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_0da95ed3_afc5_4340_ae36_7618e365b169/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_58f94ca3_e88e_43a0_8782_892cb4e200d7/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_d769f4e1_7b87_46fa_9437_6e7e7c02ece9/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_96115e3b_42cb_407c_8b46_113fba6e1059/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_872a5648_f817_435c_9533_398a2c17d0c2/datastream/OBJ/download) | | |
| Danish | da | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_d9f357f7_345e_4310_a9bc_700dcbafa0c4/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_8d5afcf4_ea98_4ef2_9622_c960cdfb53c9/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_ca11d2f3_645e_4254_b39c_87d51584584a/datastream/OBJ/download) |    87M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_ef00a3ec_3ebb_49d3_900c_4962388cc5e0/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_d5f8a124_177f_4d2e_935f_c35ba8efd1bb/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_bdd77de9_67bc_47a7_9800_1001512f76cb/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_6d1ab987_74df_45a4_9c6c_f0f3c9caca91/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_cef3fd05_6d92_4988_908d_506318b53033/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_09b015f6_fe78_47d4_9507_aac431bf638d/datastream/OBJ/download) |     56M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_968f9a25_2503_4e8a_8f3b_90c31c810500/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_86fdea87_d816_44a2_bba3_60e6f4b04286/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_968f9a25_2503_4e8a_8f3b_90c31c810500/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_957868ff_111f_43b6_bfbf_cca56ad53f24/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_382f4cdb_7896_438b_9662_a5e0675beabe/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_78c570b3_44ae_424c_84d1_bd4fa54a4449/datastream/OBJ/download) | | |
| German | de | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_49df810f_ef2f_4402_8354_4489954c7e49/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_05e49d71_d090_44e3_8168_61887d85b338/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_eb620604_a353_4d6d_9285_b607e06c2964/datastream/OBJ/download) |    139M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_9295e5db_5524_4204_9c36_7682d76b9443/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_e49e6ca4_383c_4930_a59e_57cb8eee21a2/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_2a5ce07d_88a8_4a94_8bd2_84269d9975dc/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_17217216_1c2f_48e3_b138_71d677cb616d/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_9f3b5785_d647_48d9_8037_101d6dde872a/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_d9668dc4_eb18_49da_8c72_8be6478c4bad/datastream/OBJ/download) |     976M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_89884286_2916_4dee_b73a_d43224764299/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_2fbaff9e_d1ef_4542_a384_65e5ed0efd98/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_943ded9d_c572_4912_acf2_615d79f23943/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_450c6058_018e_4d6e_b961_74e329c04cc9/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_af15c802_760d_4a67_acad_175878938b88/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_bc689f51_d29e_410c_83bf_840fb2d7dc4f/datastream/OBJ/download) | | |
| Greek | el | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_a073d92a_ae15_4563_8697_0e264fb0ebc9/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_e9e7f15d_7693_4a67_b485_6c009aee2710/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_d7314a3c_8103_4fd9_9b92_c81c16f1a64b/datastream/OBJ/download) |    271M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_54ceacad_0c00_476a_b179_c71417e9cfb6/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_7a843d7e_167f_4bcb_ab86_25e3596c73b6/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_320eaae6_f64a_4cfa_aa99_e7aec06d0794/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_f3e8b7c7_0a6c_4ae2_ae13_fd2376c1b597/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_ea27be6e_bf40_4c28_82f5_4e7613b1604d/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_3586b7a9_9bbb_48ce_8a68_b6151f7ac023/datastream/OBJ/download) |     58M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_728c473e_8ab4_4ba6_895a_2e5ea9491f75/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_ce49c1ef_12f5_4a4a_a48a_16c8f83de304/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_6e3336da_d37a_4080_8e0e_9d2eaff01956/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_8b0af8c9_ff2d_4971_93ba_795b7750867f/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_8ff2ded4_61a6_448d_be26_3aa34232f2cc/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_deb88cd2_d1f3_429f_866e_312fefa62f37/datastream/OBJ/download) | | |
| English | en | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_15b83dfa_39e1_46c4_bf97_4bc307f0aa57/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_6c6c0bf9_0887_4ae9_8fc8_e0fab7119e25/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_f3cecc3c_59f2_4b85_acba_c03bd3867a80/datastream/OBJ/download) |    750M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_53f39d94_9eec_48c6_a625_34374804fea3/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_0a061227_c45f_4728_8565_cad3ee86b144/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_fbffa925_95ea_4415_a2e0_739961378508/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_8b910258_be4f_41c3_bb43_b5a2230661b6/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_3b3fdcce_8d17_44e4_9af4_d58a5e20e440/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_ee4d31c0_c42c_472c_ab45_5d55b44260e1/datastream/OBJ/download) |     2477M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_733f71b9_7493_4e5d_9dc8_e178c07546d5/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_f503769f_58d6_4fce_9873_b975eb25a87d/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_e862da78_e3de_4733_906e_18c2095fccc6/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_d841c7ef_0e95_417a_894a_edeb5089be54/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_eda4b2e6_7e73_4d3b_95f5_0a6ee3d2b9a0/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_42ea80c5_5602_4583_b027_7eb6663c7213/datastream/OBJ/download) | | |
| Esperanto | eo | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_9f0f7da9_ff6a_4114_a757_b82d8bfc80d7/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_a6f78fb1_bd62_45d9_a25e_4438242bfd9f/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_8c221d70_c298_4b25_828d_97df5fd73542/datastream/OBJ/download) |    381K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_408e5f56_4ec9_40a0_9df0_94f4ba5bee89/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_7272b87c_8462_48e0_8996_b58190902105/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_85b17fff_60e0_461e_b158_99ab8451d92b/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_78b0aaa3_94c6_43c9_a7e1_a5e2e7a04604/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_497683dc_5955_456b_9a67_a915f36e253d/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_f7056214_2060_4e44_a48a_46ae4bbc61a5/datastream/OBJ/download) |     37M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_3a4e1112_e6a2_4498_9e57_c6934b15dd8e/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_ef458a5a_b7de_4b19_a282_1d83fd8859a7/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_1f1a847a_b492_498c_9d03_f411c8002fd9/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_f53a2c96_7b0a_40d2_8ed0_7a01d7520cb7/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_33453d66_988c_47f3_84e9_30c641ae0340/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_4a6847c4_fe1b_4421_b333_d4a91ff55834/datastream/OBJ/download) | | |
| Spanish | es | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_d8e39a33_6f2e_4486_ab81_e7e2895f53cb/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_0680516a_8143_44a9_81b7_50ac6b1b1941/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_5dbd8044_0e85_4cad_85ca_c2ef425b1872/datastream/OBJ/download) |    514M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_956778b0_0568_4aaf_807c_0025dc73e6e9/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_bf4d10b9_b107_46b2_ba7c_574ef4ae6623/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_f555f8c6_0962_421a_a1da_97d8b559f1c4/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_205f6772_8e86_49e9_a06c_c25db5500316/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_86f7dc0a_2749_44ec_aac3_a95dd6040c35/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_627fd4e5_abe2_40d0_8976_fdef13b4d0c3/datastream/OBJ/download) |     585M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_0efe4611_852a_4763_9fdd_4c94a62716ed/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_8501af8a_439f_4881_8a84_5662a622bfc3/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_3743deda_e82b_4c16_a6f8_eed7a8ca1571/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_7cebc519_9412_480f_b2a0_e27af00a91d0/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_9213f627_e94f_4024_840f_88ed7b07c71b/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_8a687c2c_5cc5_4b54_bf9c_d3a1e25cb8ac/datastream/OBJ/download) | | |
| Estonian | et | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_0ecc9814_a1bb_4b1e_a97c_163a5e870b49/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_a586272b_77fc_4736_9166_65514e5eeac2/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_8786a469_6f3a_47c6_9612_0ccc96151819/datastream/OBJ/download) |    60M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_de211859_0164_45fe_85cc_249034a5bc06/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_efd4b6dc_95b1_4ee3_a5ea_ab46e9fa3034/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_66624985_193b_46bf_9ffe_73f4043137bf/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_57bae37b_4a3c_4606_a9f5_5a8d57bea5a2/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_feeb90cd_048e_4ffe_87b7_e8066ecbd943/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_05b824e3_a58d_4026_85df_1577316c99eb/datastream/OBJ/download) |     29M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_6177e083_d69a_4743_b759_d09c3fcea39d/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_bbe9fa88_1c5e_46fe_9dd3_5875956b3de2/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_1f4a700f_ae8e_4faa_bdab_db93240ba08c/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_382a4681_6512_4efa_8b54_e7f9c0c8cb0c/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_f7eeb0ac_e657_4e97_9ae3_6ac8caf14abb/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_328bed43_0b35_42fc_9d42_dc1c6ca2bced/datastream/OBJ/download) | | |
| Basque | eu | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_8bd1d2df_3427_4e6e_8922_eeb766b87a21/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_7ea619ec_9b73_465e_9877_b2f55d77a698/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_a8d333f2_465f_4513_8b18_00d67f9c86f5/datastream/OBJ/download) |    3400K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_e40ad5d8_be99_40a9_9878_aee948ca2b9c/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_7f1fefdb_abf0_41aa_9f79_4301ee32cacc/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_1b876fcd_c91d_4436_b1d1_f6cc7b214c59/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_1678b2ce_4fb5_46a0_994c_35ab6cd2a6c4/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_251acd20_c858_45c1_8e08_efc791380b05/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_c8d4c719_179a_4098_bf07_40d556c5179e/datastream/OBJ/download) |     20M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_2eb2382c_9b05_426e_83dd_1fe733fe5e75/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_a0b1281b_b5c7_47bc_8708_4f2478478bd2/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_c7f27115_0ff4_40c0_869f_da96e23c2e75/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_f666ea17_2457_4683_973b_4b082d400c11/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_37b78937_0d16_47db_b6db_7b019d44138d/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_b1221d26_5007_4377_a710_f74f03e4e025/datastream/OBJ/download) | | |
| Farsi | fa | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_66eb0fbc_a452_4d4e_bf6e_4c0277c5383c/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_9400d285_2a62_4e3b_99ec_d840703e4ad2/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_50ffe238_c08a_410e_9ff7_e131bba40ff1/datastream/OBJ/download) |    45M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_c9088e29_0327_4ed8_a35a_3bb95ef8662b/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_024baf5d_684c_4e4d_958f_fd63cfacce66/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_3aac78ed_baba_446b_b79b_2e07ff87f760/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_a707f6db_9201_4203_882a_1dd5b2ce0d35/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_d25e1ac5_5a04_45ff_a3cb_39da6e0b9bc5/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_bee0c1c1_3d9f_4d5c_8a9c_bd13f6e0dd9d/datastream/OBJ/download) |     86M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_229458c3_04d1_4235_b5f3_71d9c073d31d/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_39576b60_e10f_480a_977b_0a1655b97602/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_9e1d12aa_e337_424e_ab82_9c44061367d7/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_67e7819a_c23a_4445_bee1_f5a9d3548b1c/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_4a262a83_e8ad_45df_9023_dee96445892b/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_4df25917_9eea_40df_937c_4ca5649c079d/datastream/OBJ/download) | | |
| Finnish | fi | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_752f9f28_053a_492e_8b2a_90e00d71324e/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_0c9f376b_dcff_4d6e_b2f7_bee580e7ac0c/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_26fedfab_3ffa_4e94_aed8_7219b43887d7/datastream/OBJ/download) |    116M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_a6ef5c52_e40e_43bc_95f6_00b714c8ecd4/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_4d84f490_9537_445c_8bf7_4021b11d34f5/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_343ca716_830c_4fee_a3f4_41a1a6c44a0e/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_0f11b209_a759_4fd3_aee0_bf59e0949cc4/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_b63aa833_6129_46d1_a98a_b319ac90cf26/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_17e8478d_d5ae_409e_a7bc_8220577d283c/datastream/OBJ/download) |     73M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_4fe6e926_35bc_48a7_914b_61ff6f6d0524/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_c896460d_9363_4a31_ab37_c92f2f795306/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_33582f68_7388_4854_a32c_ddab1f3140af/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_bb2a742e_1267_4a43_b03b_98ab074aad59/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_f8ceaada_3977_4c71_ade7_9661d81fcce5/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_90911860_bfdf_4fa9_8689_92fb907f28c8/datastream/OBJ/download) | | |
| French | fr | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_8224734f_6823_4a96_9bba_0120cb1cfd80/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_5356c9e6_5084_4137_b8e5_dfd8f6628f13/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_2719a351_e222_4365_a97e_ce92c83fe934/datastream/OBJ/download) |    335M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_5b358a31_8ee6_4c07_98b8_54dbc82a37ad/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_4b6fdd89_205f_41a1_90b5_d2da45e8cfdd/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_f64dfdac_dd5a_4b34_96be_54ec73d4e3a1/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_047670db_5a7f_4f67_a86e_644031592a72/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_903c7b0e_68ba_44b5_8c59_7c1b2ccfd467/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_8871454a_e542_4262_ba0b_7e1f6e928afc/datastream/OBJ/download) |     724M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_e9dc84ac_36a4_4cb1_8d46_557d06e6df8b/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_44104ea1_582c_456e_b5e0_a9ebfa1008b7/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_04df7bae_f36d_4413_abd5_ee7bd3219d38/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_d8e76ff0_bc8b_4e3e_91e4_39eeebfd8c2a/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_8296de12_b40e_4522_bcd1_c341843fa79e/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_439d4d1b_eaaa_427b_9307_0c64d8df259b/datastream/OBJ/download) | | |
| Galician | gl | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_8d236568_6423_44c1_92c9_5bb1db2998ec/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_54ac6eb2_4ee0_4256_b543_0867d185ee45/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_038a4099_b6f2_42ce_a257_dfd9714ce9a8/datastream/OBJ/download) |    1666K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_d29a9e6e_d45f_431e_bf3c_8e695804ce3e/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_73d8b2c9_bce0_4b0f_b7c0_09da034cd4dd/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_afa136cd_f883_4a6d_9106_b0a658000555/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_99432292_6faa_4859_b445_8070a33a6221/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_b737acb3_00fa_4795_a71a_c875fdc26705/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_0d26af32_7301_4c3e_be90_8dfca413e6f9/datastream/OBJ/download) |     40M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_0696704a_73f8_44f2_943a_b370c00b2e3f/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_ea3cb41a_4002_40ff_b300_0b183336d8a1/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_055c001f_a61f_46ae_9cfe_d987def0691d/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_d527af03_cb0a_476a_8099_ae49ad5df472/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_58ba3c89_105c_414f_b999_00423611e0c5/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_a3580f64_1803_4f52_8548_01f0ef72008c/datastream/OBJ/download) | | |
| Hebrew | he | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_bdc2c704_bcd3_4edb_b33c_f81299615af3/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_fbfd91bd_2db2_4a6c_9337_85d00b0d54e9/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_a2cd4066_05cd_4640_94de_80dafca4f6ac/datastream/OBJ/download) |    169M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_888dab90_00f7_4e99_aca7_38ec00a6abd5/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_a988de79_31ab_497a_8c31_c9cef9b29ced/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_80d9a744_38db_4413_a19d_e9f059c7c33d/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_8652d4a7_03e0_48d1_a3d3_a81b5f02c0d3/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_4d3e60c3_5384_4927_a7f6_a11fb8d078fc/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_a594f4da_6424_4c1a_8a8c_b4c669f311b5/datastream/OBJ/download) |     132M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_5d689f29_5be3_455e_a71d_b39e12f988a0/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_dda6fab5_8db8_4e68_aefd_4ee237113e8d/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_29ed95fa_14c2_41b1_afbb_4092b47dfd4d/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_a0bfc84b_417b_448b_b9c6_c05abc56e0fd/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_4dbda19e_9ed3_4f9f_9dd6_9bc58e777f94/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_c1113aa0_b208_461c_8b9f_26bf19d68101/datastream/OBJ/download) | | |
| Hindi | hi | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_448f9995_6492_43fe_b9fb_14a63fb9d24d/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_004efa6c_d23c_4cce_a327_994f94e59437/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_4d930c9d_ef9d_482d_8796_02efb83a33df/datastream/OBJ/download) |    695K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_d27adde0_49ef_473c_9239_d93be7f4805b/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_d5dbb722_97ee_4a21_838a_122d41bd3fcd/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_7ffd12b0_49e5_458b_8541_839a4a91408f/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_a0a794f3_1d87_49ed_af2f_f306e06e073d/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_9ffcadfe_6a73_43ca_be7e_131bb581ab1a/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_639cd70a_0a7d_4bba_9ac5_20f5ee1af0a3/datastream/OBJ/download) |     31M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_28785bd7_fd3c_4a40_8c4f_43123c62f852/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_ea23e53f_c734_4148_a5f5_43785e7ff5f2/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_da4c1867_d44f_403f_bd02_09bf3e717ba1/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_dd318b5b_7d39_489b_b630_3e78436ecde1/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_dcae62c9_886c_42c2_8d8c_d20b0c3fccf6/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_c060244b_6c7f_4c04_b690_3691f9eecaa9/datastream/OBJ/download) | | |
| Croatian | hr | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_5e5185bc_6308_4c3f_8d31_baa250a33729/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_809f3504_9748_4e5c_8a1c_619b05ec91e1/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_8a373fe3_1931_4709_8b45_bfffcc27e91a/datastream/OBJ/download) |    241M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_884cc1b0_3d5d_49cf_b147_a131a94fa49e/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_d57cc8e8_18c4_4f0c_bacb_6555751db391/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_984e21ad_30ff_4340_9e2c_99b503e2c592/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_7dce74dd_665c_45a7_9111_9b0e1df45862/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_f9e4e167_2e99_4454_a3ab_8e85dcc805c6/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_b15c4464_df3d_4284_9050_270a12f4e3fe/datastream/OBJ/download) |     42M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_d35105a3_0d94_43d0_965c_5b7c8deedaa3/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_3c0494fa_308d_4dad_87ea_7744bbb4fa87/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_a1dfe60a_162b_4f47_a89f_d1415f1f490f/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_551935cf_52db_4214_b9ab_f715bb5b1c13/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_93f62783_b04d_4782_bc87_595e05fd5fba/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_ebcb48c6_31d4_402c_b30c_41f290692a8b/datastream/OBJ/download) | | |
| Hungarian | hu | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_3e320d72_7b08_4c93_99ba_979b6db2ff3f/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_0128ed6a_e86c_4892_9694_0c3430796945/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_c158b454_8c17_4097_9fd9_e7512e9983e7/datastream/OBJ/download) |    227M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_3bfe5491_a955_414e_92ee_2474009a41c4/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_fecf31df_ef3d_4470_b864_b64c81ef591d/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_0d2555b7_27d6_4680_9290_e3cfd960f2b2/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_f39bf640_c0fd_4630_870e_cfdb20176b3d/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_bead1624_8f8a_4bca_af1c_30218e5e9376/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_93526198_c43c_43b4_9e0b_c90562a3d136/datastream/OBJ/download) |     120M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_1bafc943_ba55_4594_a5ea_6fdcac264b56/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_0929b4fb_83a6_4935_8230_8518c7dc39c8/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_b998b0c3_ee4a_4b68_af42_90b9005983f7/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_8440b731_1972_4b79_bd30_76103fe05b87/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_5048590a_58f6_4627_8f76_83a176d92864/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_68fd7e68_6f31_4480_8aa5_ab4ee8f1592e/datastream/OBJ/download) | | |
| Armenian | hy | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_36d2b3c8_bfb0_4b98_8ebe_3140b8b1b21c/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_af5721e8_9841_46a2_abc3_3a0b6ce28e22/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_614bab22_6e09_428a_8a2a_1ccdfb1f45c0/datastream/OBJ/download) |    23K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_73a1f725_50a6_475a_9ed9_86ef6564495e/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_dca3fedb_38e8_4c9c_ba0c_0ec5e0309780/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_0d02b0eb_d3ae_4702_adde_aff9df0b1740/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_161a90ba_474d_47f4_a734_2a4b8710848d/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_6df26e68_29bf_44c9_8c2a_f223e5c49e19/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_73dc0bf9_280d_4f2a_8937_8ac17d6bacc1/datastream/OBJ/download) |     38M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_7eb0dec6_8bcc_4e44_a1e4_a876d5ad7111/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_34e88586_2e74_49a9_a32c_3a336fe5168f/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_452e1081_6330_4e8d_8fc0_cdbc6a899250/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_bebbbf79_caaf_4590_a33b_45068b97ac7d/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_6599d23b_fc64_4b47_9bf2_7dab34bb49da/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_6c467a0b_d991_48d4_ab05_8f387790f35d/datastream/OBJ/download) | | |
| Indonesian | id | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_51c31a20_2720_4541_9640_0c94f301fe79/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_e8a44d22_24b6_4a53_82c7_76858cd0652b/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_9f48889b_e9e5_485f_8262_0ce78c72a67d/datastream/OBJ/download) |    65M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_1b5bccca_16c8_4b3d_8f02_bd8b80609796/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_b4e2bbf7_26ed_4382_a0fe_9c9a4e3bb082/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_6bdff073_5371_4dfb_bc0b_d25c96b53230/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_3ab72256_7952_411d_8e64_a2d497745823/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_3768e4e5_5478_41f4_9c8c_9ab566a69afe/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_33c28bfe_b6f9_4270_b205_625ff863b4bd/datastream/OBJ/download) |     69M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_a45cf1c9_76e0_4f80_aebf_ca1f500f0a35/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_3c5efe05_9382_4564_a8a5_90e77d918257/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_88a6a53d_a647_4512_830a_90e2b3613c53/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_e8a44d22_24b6_4a53_82c7_76858cd0652b/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_8b027265_58c5_4b84_b6aa_1d1243ee974b/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_e2ddfb71_8a3d_44d7_ba71_f98a01cfd9e0/datastream/OBJ/download) | | |
| Icelandic | is | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_d3fe8447_85a2_4cae_8b50_aa21caebf3f6/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_254a19dd_6f43_470d_84b9_b4973605cb77/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_6863d1b4_8c2b_490c_b56f_e1c99be40d67/datastream/OBJ/download) |    7474K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_65e255d4_80e2_4fc9_b7c0_7469186324c4/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_e9f7924c_fe8d_4d06_8bba_ac4e516c7017/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_6728b867_a11c_43a7_b8d6_cbf7f8894eed/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_8585a2cd_bc60_4f8a_ab2a_d0a957164a09/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_cc92a537_c7b1_41a9_83c8_e39c3fd425e3/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_da5a2064_104f_4f78_90b0_17c8df08d4c7/datastream/OBJ/download) |     7196K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_aab80718_5572_4e9c_9898_69ea55fb139d/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_b16032a9_d529_4ee7_be6a_f6fc352f61b6/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_7f0fff6a_55a3_4596_970e_95001b1b0b64/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_555e0196_16f3_4705_89fc_0a243e2c4ae4/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_3f4048d2_d53c_40cb_b9db_709bf2d3802b/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_a83b7176_6d83_4c37_b810_402bc78878a8/datastream/OBJ/download) | | |
| Italian | it | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_68a7bb29_5b0b_43d4_983d_2bd101526d55/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_8c93019f_eb8e_429f_b5ff_94064316abb9/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_09c0756c_e465_4311_a1bb_7d2a5111f4cc/datastream/OBJ/download) |    277M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_168bc19f_4197_4fc7_b049_fc840ead2d40/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_6f0951eb_bdae_4d7f_855e_28a1f2a15865/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_ee07fc19_351f_4fa2_8d62_01ce775de95f/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_54186514_c2bd_4ab7_813e_af56d0639ad5/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_92e0f3be_fa9f_4653_bc6e_024ef20fca42/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_e4c035cf_8a29_436d_b30c_c11397cd791a/datastream/OBJ/download) |     476M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_7720dda8_98a6_466d_8ea3_7ee9c052900e/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_e7f3adaa_4e68_47c0_b5ea_176cd11078ec/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_5b23d792_541a_4a68_b1ae_e5220a3e1973/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_ee3b96f1_a21d_4e86_9e8d_7ea1c3b18ce5/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_1bd39f98_5122_424a_9647_21a806b3ce51/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_9cd0d7c5_3e5a_44c8_b4e8_ca9c89de35a1/datastream/OBJ/download) | | |
| Georgian | ka | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_a62063b9_41cd_42c4_80ed_5c479284c106/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_4cb72516_aa27_4bb5_9f88_ff07ac9df34c/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_4a874afe_7296_41ae_8b7e_0ad455adac90/datastream/OBJ/download) |   1108K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_e4daec16_2b2c_4ecc_9e3e_782b009da4d9/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_a4409772_5507_4983_9420_bfac6d989f7c/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_613c81a7_8228_4e4c_bd54_93d993b39e46/datastream/OBJ/download)  |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_163ff85f_ee9f_4f70_8a4c_5502cee227ac/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_96db3a5a_b699_4137_baea_4665548d1476/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_ce9c0a1f_2fee_4c00_a413_a94f7c74f718/datastream/OBJ/download) |     15M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_a825e786_d920_40e0_a17e_c98ae409d449/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_b6aa9a1b_ef40_4d80_8664_bf5b815ad0b0/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_2081d5c2_1b5b_407f_940b_363c7c573f13/datastream/OBJ/download)  |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_0126dad6_9f4d_4002_a918_07f67a696300/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_81ce3dc3_62de_4bd6_bb9c_b8f792b6bd3d/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_869d6b9a_e36e_46bf_9e06_5cdff0e9f497/datastream/OBJ/download) | | |
| Kazakh | kk | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_44ffc57f_2907_4a6d_ad8d_5cd2dbea548a/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_eb1c5733_9db5_4704_ab4c_5902f70fccb4/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_d98342a1_0d67_4f94_9b33_c7e475ccdb38/datastream/OBJ/download) |     13K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_73557531_5ebe_4fd5_aaf7_96a328f3288c/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_b6bb4967_e417_400a_9abb_5dc3e8dcae38/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_ba79d053_a8ee_46d3_9e7f_27eaebab9cb5/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_850e9909_d908_43ba_bdbc_762cfbd86cf2/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_8c3b1f6c_576c_4075_88ab_25cc4070f2be/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_1d277b69_8535_4eb7_a4e6_9c30b418a537/datastream/OBJ/download) |     18M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_40bacff9_818d_49ad_95bc_a550fc3247d4/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_7ec6488c_21ba_4109_af84_4c437bf22200/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_5d0dd48b_6b03_4ae1_9136_526485003675/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_56778c23_b5c5_4386_83cb_29d04ad014ea/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_b7a5b672_edc3_4254_8759_98eb6404f8a3/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_b1350e33_4e2f_4288_98be_5fdacef9b00a/datastream/OBJ/download) | | |
| Korean | ko | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_bbac0453_6c95_4c82_af67_f4909720d578/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_9bd14106_e852_4ba2_a523_88a71e2fc1a5/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_903f9539_86f7_4521_a2f8_2355f7c95a2f/datastream/OBJ/download) |   6834K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_76a0c30c_89ea_4fd4_8eeb_e671dc855edc/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_959a996c_2796_4168_93ef_b6e410aa0a3c/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_1268820a_d32d_476c_b5b6_060914c1ea21/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_e2a992e6_7cd5_4f7f_b8ac_ca17a50eae3c/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_0b635218_eca2_48f1_b2a5_260d023681f2/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_8068db80_6934_472d_8bf3_97089cbc40e3/datastream/OBJ/download) |     62M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_f652d0b2_6653_4839_b04d_5615b8333f91/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_a572deb8_089b_4213_ac5e_e924429db33a/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_ebec81d9_4ec8_4454_ab14_d540693f720b/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_7df56c99_9d28_4cff_b451_cfaff102ed1b/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_c6c95a97_8947_4632_9585_f29030c21e17/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_dd0ee9c1_5abe_4112_b753_68427c4296eb/datastream/OBJ/download) | | |
| Lithuanian | lt | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_f76df11e_95fb_497c_84ed_e23e2279e2e8/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_01a0e0af_d1b3_4a72_a221_7b1dd6833b14/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_350358e2_e6cd_4187_a837_652f22736bbd/datastream/OBJ/download) |   6252K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_50610f54_6ce9_4a3d_aea5_48ffdf8f0fc1/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_f92815d5_6cf7_49ef_ae29_f969c883dd5d/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_d737ae7e_c8d9_4442_a46e_15381ba66c72/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_e45287ca_f7d6_4ff8_af39_89bc903c9d55/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_aeb3c6f4_b2d3_4570_9b99_617932f6ce56/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_1e26774e_df35_4226_9d3b_4b2f0c6a845e/datastream/OBJ/download) |     23M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_de604796_0112_4651_9cc4_24f7c6e7996f/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_ca300706_672f_41f8_9302_ef18cb860cc6/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_957139ee_80a2_44c4_8c6c_70d3ee755d07/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_c998ccca_a700_40ec_9332_25b4c6dc4363/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_eed538a6_666f_4bb0_9938_df8a79619203/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_49d1e6a5_48c0_441b_98f2_0d5c632331d3/datastream/OBJ/download) | | |
| Latvian | lv | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_f80e4bb8_86a4_469a_986b_a1c1bc49a973/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_f621c77f_5b94_45aa_846e_470631a46c0d/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_ef8f3a4b_137d_4343_9341_30fea39af270/datastream/OBJ/download) |   2167K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_6a91fd2c_c417_48cf_bfe4_7317e95d8a2d/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_4ff957f0_2846_48db_89b0_b046daa381b7/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_898d63ae_ca84_4993_8914_77c99cb3de8e/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_592fabc7_6ecb_43ec_837c_e940f479061a/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_7b2a566a_3361_4f9b_b6ce_97e69980b48c/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_0f2d42fd_f4cc_4a49_a420_832a72f0f61f/datastream/OBJ/download) |     13M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_ee0c59d6_8861_4805_b865_04f8d6187430/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_230b43c0_1cf7_44d6_8328_230128dede00/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_6d24d3f6_49bb_4098_be10_4b2ea6e3195d/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_0de41671_b1d4_41ff_a86b_3de4c28d782a/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_886bb411_9154_4aec_a208_675ef3c06e80/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_a9af274c_541d_44e0_a0ac_c983ba9390b5/datastream/OBJ/download) | | |
| Macedonian | mk | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_fcb27857_3c2a_403d_bb7c_8f43ca0224ad/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_2cb685d6_ce0c_4f4c_9253_283b474a5f2a/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_180b7221_8948_41c9_b75f_3a38e8e92e18/datastream/OBJ/download) |     20M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_faff5266_9f41_49d7_982b_643b8466782b/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_37fa5ace_cc32_4664_a592_b7d1aba46670/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_8c17fea7_410a_4c17_a289_74e2576baa73/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_16ecfcd3_2900_42bd_9f97_c5dd2bd1019b/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_21df4ce4_d137_470d_a56d_c55bdd1e3245/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_0d998d85_5c31_4276_95e6_ddd1ac27a3cd/datastream/OBJ/download) |     26M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_b21ddd7d_f887_413e_8995_2aa57498b3ea/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_8dfe1739_2504_4a0e_a44e_8d7612c76e86/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_ac8779c0_4dc9_4545_8a3e_2f7256dd488d/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_7dee4f33_5979_492e_a5ab_888d28245dbb/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_85735ca3_486e_4e21_bd23_fc39eff4a6b3/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_df65508b_fd67_4524_ba5a_1e99fa13ebf4/datastream/OBJ/download) | | |
| Malayalam | ml | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_f6c6298f_aa9d_4dc6_8fe2_e3626d674cbd/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_3921fbc6_a509_4028_aaf0_5b31cc47e62c/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_73cd8422_228c_4ee7_ba97_c65622e3baf1/datastream/OBJ/download) |   1520K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_1a0b6c67_30a5_451f_8b95_2a0da9c3070b/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_2d1282e3_63ac_481b_95ec_de159294df64/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_98c5650d_d771_4b44_85b5_ea9688da1778/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_addecc73_8134_4437_8ea9_dca7628917b0/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_251223b4_6875_4d63_8dc8_1194360a1d86/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_04a501f1_84cf_435f_843a_c8168d789fb5/datastream/OBJ/download) |     10M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_518af478_4f8d_47ea_89a2_b8385eab3ce2/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_e38201e2_c6a1_4ad7_a6c4_55b5fc3950ef/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_9a7c97f1_ae65_4c91_a01b_ffd147d4906a/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_4e5d035d_f5ae_4a62_8604_c12d5e49c629/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_70b69424_2fd1_465e_ba4f_f3750879be8a/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_f77cbeb4_ac6b_46a1_86ca_bc259899b1b2/datastream/OBJ/download) | | |
| Malay | ms | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_8f781662_4818_4ee8_8671_3ff530e94895/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_f70f8025_ca62_48eb_815f_ef2af8e96b54/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_eb497882_8791_4c6b_acfc_220f1ef7f864/datastream/OBJ/download) |     12M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_a207ed8a_32ee_4ddb_97f9_7014317be1af/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_42090298_6f17_4951_b009_72f268253703/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_07d972bb_5395_4dd2_be62_2573f1af3f50/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_35dd9578_70c1_4812_b231_d4d8f3102b9f/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_296f0bc9_d22c_466f_ad74_43c83aaba658/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_6cdf09f0_74f6_4a18_bf24_4f7ed8a5bccf/datastream/OBJ/download) |     28M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_fb0105f0_3767_4c7e_bfd0_7d30a2dc2a98/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_f595969d_6cd8_416e_ae83_00d5c67cb067/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_87435115_db79_41d9_aecf_6126ef80f1d4/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_02798b0d_05d6_4f54_b781_6f4946fe1f84/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_cbb71d23_2bc4_40e7_b590_e16d3293ee99/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_435a53d1_972e_47b9_9e3f_196cf3b099d1/datastream/OBJ/download) | | |
| Dutch | nl | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_90f21a0d_b28f_407e_ace0_c0f81be08367/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_b6921635_11d4_44f1_abb8_cd767bf4e091/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_d65f47fd_d102_4d5e_a947_afb963af1244/datastream/OBJ/download) |    264M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_06b54ceb_2b05_49b5_8bbb_26a3a21d2e68/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_41be155e_d2b5_4142_95bd_2255f5432edd/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_2d2e0374_5bfa_49d0_91dd_1ba44027647c/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_eeb6636c_ec92_4ea0_b198_e6ef07cc12d0/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_70da1b7e_b71c_4778_acdc_5f48ece36276/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_7a21000d_fa22_426b_8cbc_06bfa46035ba/datastream/OBJ/download) |    248M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_c2c50969_0f8c_4dc6_9ba7_2af8f153fea1/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_dbc0da7e_9202_495c_8c2e_e8e4d87537c1/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_f1e0c3b6_0bc5_466a_93b6_1358b4923aac/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_156071ee_46e5_4e2f_8043_635947567261/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_aaf3950b_0e3d_4098_bbea_8c7f598d43d4/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_2812b150_db64_4aa1_b642_40afeb41885e/datastream/OBJ/download) | | |
| Norwegian | no | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_3d925474_7d0d_44d1_8450_098d974dc961/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_21df17f3_1da5_4c8e_a47a_f9a2b69b283e/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_92522ed7_33ad_4872_812d_1521f2c6bd44/datastream/OBJ/download) |     45M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_6089012f_91b2_4e96_b951_bf18b84fb74f/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_388ae8f9_5add_4742_ada1_abf78d39d55e/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_6928a125_1b6f_47f4_a0d5_ba9e946b75a4/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_4773d2dd_9a0d_4f29_911b_b54a4d3541c9/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_55be427b_8e78_436f_8cac_8b3d8c4368b3/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_8330f9ac_854e_447d_a702_ee09873d925d/datastream/OBJ/download) |     90M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_8ead35cd_8b49_4c9e_80ea_59de857d8dd3/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_d81f2a73_b92d_41c1_8056_31ce246df4fe/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_92049562_125d_475c_8e2c_59d63edfe50e/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_bf51abca_e8b0_49c0_b522_d340cb08f640/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_87aa65f0_4a94_4553_a9fe_7c8647c5a08e/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_7c6a8309_cd27_4e92_b69e_eb729e23fb3d/datastream/OBJ/download) | | |
| Poland | pl | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_515f5a41_5bdc_4fe2_b330_13d9fea2e571/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_6ec2b5eb_8195_4fdd_8d6c_ae8f749fb77c/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_7e19c4c6_7b4c_4c20_9f67_7b6812075351/datastream/OBJ/download) |    250M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_6ad738bf_95d4_450b_809d_382542d5dbdb/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_c507abc7_9604_488a_b086_8bb08428c712/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_ec446e1e_2cc0_44d3_9102_902da99dd8cc/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_488f2553_cd9b_44ba_8b68_815ba8e0e6b1/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_753d840f_9472_4b90_9821_9b206d83a889/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_c76dd162_179f_45a9_a3f1_75b72f807d3e/datastream/OBJ/download) |    232M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_3835ea59_db1a_4518_85e0_dd0b88ecdd95/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_e105b170_8fe6_4004_891e_a8e79fd1dfff/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_d6c08d5c_d4a6_4267_862e_0a29ed6bbd1f/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_ee1d3625_37ec_4fb2_ae7d_69dc04465433/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_b52fa3c0_f856_4c54_8e41_1aa3f3cc2976/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_bd9670f1_19cd_4635_9fc9_5b42e01ec737/datastream/OBJ/download) | | |
| Portuguese | pt | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_bd1f94b3_f7f1_4117_990c_94ad48880ab5/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_75efab4b_0596_4128_bf5a_ccac31acb33d/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_9ec62f05_0093_4a04_818a_a5b96c15a102/datastream/OBJ/download) |    257M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_bce03abc_13a9_4798_a642_8b3e3bd725ad/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_8c866e8d_90f0_4cb9_b013_ddd990f49a4b/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_505f257d_0aa0_400d_9255_af170a9cf1b5/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_e28e7be8_1a82_4b5b_bdb7_d5aec0494b5f/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_5cc9a32c_c84c_42e8_93ce_3eca2c98563f/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_2f0d49f6_1f46_4603_91e5_ab1d8637ac10/datastream/OBJ/download) |    238M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_3ef1436d_b353_4e45_941a_4e2738436a72/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_a78ede2d_304d_47ba_967f_a5484d4cfe51/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_76877729_dbc2_4c7e_9128_c1410a3cf2f8/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_1e4f1919_6ef4_403d_86bd_10c31b874a41/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_9b076384_7f2a_4c0b_b388_da71aea8fb2d/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_6a141fae_4095_469d_a3d4_7e5dc8883997/datastream/OBJ/download) | | |
| Romanian | ro | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_6c23d90d_2c7c_4f9e_9879_363b1dcb5b2d/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_042a07f0_c772_4479_a04a_ea6a139f7db0/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_2276ef0f_a563_4981_b721_0d4dd010e965/datastream/OBJ/download) |    434M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_8a99b75d_feb7_4386_91e3_6f0ddb3a5449/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_d39643c3_5e6e_472b_98ca_8552951899cc/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_2b180814_04fb_4003_812a_5fbbe29befa0/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_df76e8f9_d81b_4a01_b35e_9eae7be6300f/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_6dc589d5_910a_4d2a_bb40_a079ff624f34/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_526e1221_78d1_4157_97b8_02e1422df0fd/datastream/OBJ/download) |     65M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_82c7a318_5892_41a9_a760_5456a2930dc0/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_97fcdc67_d33e_47cf_8507_0d1075658f71/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_af718433_b54c_4edd_8628_a7efafcbbc7f/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_95a96676_7615_4a53_b4d6_788622b1da48/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_3bbbc032_67e0_4690_9a85_d22cb02e2c37/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_6d5a207c_9871_407c_b07b_abd35245af82/datastream/OBJ/download) | | |
| Russian | ru | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_dc3d584c_4b75_4922_9cfd_bde933cbae29/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_819d6423_5ef0_4b1b_bb5a_481544cffb50/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_30660859_70be_40a5_b4a5_7795e70502b6/datastream/OBJ/download) |    152M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_83923d5f_c9a2_4829_8084_839a1b71db0e/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_d4b17371_de03_4ac9_bb69_62fe0f260640/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_e34daacf_e39a_4258_833a_0f39d461427a/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_48177979_ac17_4c77_8083_09896f80f060/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_a4d949ff_287b_46f2_b420_47989f833b59/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_812f684a_780d_47c5_b712_9d3e7c218fd1/datastream/OBJ/download) |    390M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_bd327c14_c505_485a_95c8_b25679dfbee3/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_f2c7bc3c_0013_4c07_b40e_f168a3dea61f/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_7ac757ed_e2f3_4d70_a8d7_e90e42d36e77/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_b4c02f28_5d54_4ae5_bb85_63515e7c8df5/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_4dc48b41_f39c_4390_af1b_bb569f6ba26c/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_8c7b9d0e_4ba3_41d6_aa72_dea8ca15ea40/datastream/OBJ/download) | | |
| Sinhala | si | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_35e5500e_b802_4e7b_9ad9_aabb85872eb7/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_0c17af69_15d0_4d7f_95b5_b16fced4787e/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_c6db28bf_eadd_4481_b4e3_ba7642530ac8/datastream/OBJ/download) |   3493K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_5ba52a14_4718_4626_97fe_b6933dbf64ce/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_1fa68c03_fec3_46b1_89c7_f4e174632746/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_c1b9e68d_76cd_49a7_b52f_5606f0140a15/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_130368a3_5af1_47e0_b675_2afda4726894/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_3e9cdaeb_58e3_445b_9b6e_39372fd856a7/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_dbfec0bc_1a5f_45f5_bdb2_6179879c2c7a/datastream/OBJ/download) |   5980K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_8eb0854e_1ed9_420f_bb02_1b20ce49c6b6/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_50cfb753_3052_4607_b763_9e949fea5738/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_49315977_e174_41cf_b288_ecf1a40da2e0/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_9f5bfca1_79be_490c_96f1_13cf87f04de7/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_5599ee86_1d4c_47d6_9c7b_da3c59b1edf1/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_0981e04c_825e_4856_8bd4_05cffcfa0195/datastream/OBJ/download) | | |
| Slovak | sk | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_9a6b3734_a470_41dd_a12d_613af547dccc/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_099c0ceb_c695_4515_8398_85b3e470c34e/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_26a29e9d_50c7_4f88_84f3_590dd2df6e3e/datastream/OBJ/download) |     47M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_43d0a976_3f0a_41e6_8d78_7572cd60b5ad/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_42d9fb12_cbbf_4db9_a4a9_04ed66944b2b/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_4351f8a2_b05f_4aee_851e_03af7d081e5c/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_e593b44d_78cf_4abd_b93b_69cec434681c/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_ba6ccc04_9e86_47a0_b698_185650789a4f/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_206854e5_70df_4848_8a53_b21fa53223e9/datastream/OBJ/download) |     28M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_67659c83_942a_4144_b1cc_0c173d068156/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_6ebbecf4_57ce_434e_aa15_5c122c572196/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_72c7a6b8_9ad9_4626_b3ea_7cfe9bdd1d5b/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_f7fa40b9_7654_427b_a81b_ebb7ece9046b/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_0588b4b4_1ae0_4b1f_8476_e61cee1a90e3/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_468de577_6c42_434f_a0da_bf1822716aae/datastream/OBJ/download) | | |
| Slovene | sl | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_82430fad_eff4_44d0_bed2_ff9828128062/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_bc4cbfb8_014a_4d31_92ba_a8aaf1b2f9e8/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_6d95508d_a32f_4b37_a7ff_a4bf3559b9b5/datastream/OBJ/download) |    106M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_13771847_f7c5_45f4_9d8f_f14c95c90ddf/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_c8749bb5_b0f3_4ec6_a536_6276d518afcd/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_182586e9_d6bc_4a2d_a3a4_d3eb8f349dba/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_e184679c_a0e9_4412_b5ec_4fc1b5b76c80/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_a627f276_6d9f_4c8c_9259_11b0d1141dc7/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_04c6bfd4_d16e_4366_9bb6_febb53b44203/datastream/OBJ/download) |     31M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_f7c4c734_2a20_45b7_a864_901413931e07/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_bfc18774_eef5_43ef_b483_0b6f366afdc9/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_e14a93d0_8dd9_4f2b_92bf_3196934078a2/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_ab542560_0ac0_4b54_bd8a_37586418228e/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_ae27eb9c_b5f3_4a95_9f41_ea4e9f9c5aa1/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_3f819ef5_cbd6_48ab_8851_6cae10474700/datastream/OBJ/download) | | |
| Albanian | sq | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_1646408e_44bc_4b8c_abcd_605238659680/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_3fc64b62_5f37_456e_b5d4_f9b07b605097/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_3e714c9c_8ef1_4ef3_94ef_87a6e3dfea82/datastream/OBJ/download) |     11M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_3980364f_6a63_49b2_9148_43adea2ce8ad/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_f4c24add_77e6_4fa7_8688_fa3a58fc660a/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_7ceea543_bfc3_4ec9_a8de_d9017141fb83/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_4298e6c1_eedd_4bba_8e29_262067ffe142/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_2b78a765_e4b3_454a_8e98_01f891bca373/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_8d367f29_0cbe_42de_9f76_cd3358b002a3/datastream/OBJ/download) |     17M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_658a7ded_f7bf_454d_a12a_e389cfa38353/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_7ba7aeb0_1c6f_46b7_b427_b1278ac12a3f/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_beb35b4b_e91c_48de_b6e2_eb7dcd42f1af/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_f359907f_8459_4567_a1ce_232c784c9c63/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_b879ced8_6c9c_4044_bfcc_ed48f70df052/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_f9f27458_e612_4cf1_a24c_d9639b9c1fb8/datastream/OBJ/download) | | |
| Serbian | sr | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_f738a84b_a120_451d_87ff_0de74b37dc1f/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_4da359b5_b004_424d_8ef7_185e373ff282/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_4aea8798_be02_4519_a1b9_04c99516e465/datastream/OBJ/download) |    343M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_ea8297a0_f529_4baf_bc53_6ecfaba51a11/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_978bf81a_5747_4ca1_9404_9f4e3721a932/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_a75a2279_605d_4a74_b7ee_920a3537d4d6/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_1ad6f881_29a5_4a53_bf81_9526dad9b15e/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_7ec578ab_49d1_40b2_b95f_8b3cc7a00e34/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_bc379acb_c3d7_48dd_8c8e_6d75fef9dd47/datastream/OBJ/download) |     69M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_faf30af2_045a_4563_bcb6_f13c8f0cf850/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_c3cbcd2c_4bb2_4f28_abad_4e6b5f9a708b/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_4946db9e_3db6_4a2f_96ff_50e488bc6c89/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_0390eeaf_0785_439c_bdeb_94092d1f9bbf/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_102ee0a7_6479_4e13_8399_f816e5d1f4f5/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_9840d623_4b71_453c_ba78_55c81b9242e7/datastream/OBJ/download) | | |
| Swedish | sv | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_c2d706f1_dcb3_40e5_bd6f_0662290eff86/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_708dbb78_83fb_4264_9d03_a208890e621d/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_95145a24_7afd_436c_b64a_dd3f262c5e11/datastream/OBJ/download) |    101M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_0a7fe79e_c5ba_4c2c_9fff_38606864e675/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_6b376707_085a_45d2_a156_08702fadf7cf/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_d5d6d279_3b5e_4dbd_989a_6fe683cf02bf/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_5c62c04b_56d9_4ee2_bc47_82b7eb3ae044/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_c40ad13e_56c6_4e34_a151_ff86dc7a787e/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_0e7cf2d1_3de7_437f_84f0_dafe5eceb143/datastream/OBJ/download) |    143M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_60a29f9a_7a29_4ef5_a1a6_a24bd7b4423c/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_869987f5_6781_4654_9721_6e5eb0ac05d1/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_2fa3e781_48f7_4de4_b1d8_35a74e44f56c/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_ffdedc93_5eac_43c8_9f37_ca4d5593a08d/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_76351cdc_16e6_4332_bbf8_a92a4415fd20/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_1edef66f_3c6f_46d8_b8dd_d51c77457646/datastream/OBJ/download) | | |
| Tamil | ta | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_9f66ac3b_006b_415a_9f76_62bb8812d320/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_f38dbe7c_7e57_4512_bae8_6badaa5ea639/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_557220ef_5510_4a0b_82ff_dd84b05b6857/datastream/OBJ/download) |    123K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_9e8ec965_ee3f_46cc_9e96_3bfc952a75e0/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_e0dc15ae_c3e8_417e_a42b_5d4b3b098694/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_6dadf675_50f7_4249_9f07_22350f227ab8/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_ec6cacf0_d7bd_4bf8_aea2_3c54d12e966e/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_b874e2d9_7b72_4f7f_ae77_de84a2dcd9f7/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_b2efcf31_605e_4e20_bf3c_9d8321d5dca2/datastream/OBJ/download) |     17M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_7ac79d70_fe38_4c8e_8434_0cdf4119d734/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_00e6da45_d56d_4698_9130_82a555e6fe97/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_ca957d15_6b32_4305_b12d_0cc1ca2fc84e/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_1078deb2_855f_4dc4_98dd_4c0862e34d90/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_9c5b7b79_16a2_4e39_8991_f0f8de686b7c/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_270bef3c_2777_47f4_9a79_869384fe20db/datastream/OBJ/download) | | |
| Telugu | te | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_b30cd31f_433c_44b6_8aef_c3708a806b7f/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_57f3b730_4341_4a25_8a7e_edcf2496bf73/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_74e479ed_5ccf_4bbd_ae58_259ff6f24e74/datastream/OBJ/download) |    103K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_25edb0db_7cab_4839_a282_6c9da21c172e/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_e3bc1022_f09e_4eef_903d_fef503e8efa5/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_486a7bd6_f48f_423c_83de_fe7add2798c8/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_716e7b3a_5423_4380_ae00_43650c3a7313/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_4fa8a696_0d9e_4cf3_a2d5_dadf9782bba9/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_b28d2cf4_4db7_4cde_bb25_f74d37bbd704/datastream/OBJ/download) |     15M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_244c3038_7e44_4548_8dd9_028dd7ce7a27/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_47b6b36e_913b_45a6_a947_d68b179a6212/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_6901a2f1_2a30_488e_b6b4_ff12079b7163/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_af4dcf21_876c_4e71_b4f9_3a6be875f3e3/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_773f0849_affd_4928_a5e2_a8fc0bf2fbf9/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_4daab0f6_4a46_4711_837e_c61c2c99a06f/datastream/OBJ/download) | | |
| Tagalog | tl | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_c37606ea_d722_4642_85e0_5ff1065617fd/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_42dbed79_ff03_462c_9d8b_13db02ca1f12/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_1e64cb87_4e4c_4959_acf5_92d96e775578/datastream/OBJ/download) |     87K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_752ec456_bf90_446d_9ade_e790486b232f/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_dac911f6_9ea0_4afb_ab59_c7af66322c79/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_74e25a0f_f2ca_4502_a876_66e94c798586/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_2bff1a02_6f9b_4d5a_bbff_d2037b1739a5/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_894af728_d65a_4d13_b3ca_bf765ce7746b/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_7134367d_bbd6_4f55_8784_0725ef666c97/datastream/OBJ/download) |   6515K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_d6a420a2_ab17_4fd2_aea5_66fe2c5e0464/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_651f0462_1cad_4c9a_8c09_11beab533d74/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_0ca47f8c_66fc_4545_853f_4b6ef4e93a4d/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_9171350a_d626_4e26_8e65_6400d0b414e5/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_2b5b7945_41b2_4ea7_b720_c34ad35d5a2b/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_da8bc801_44d6_41e9_92bb_6009ea1d6835/datastream/OBJ/download) | | |
| Turkish | tr | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_89c314c7_c0e0_408a_ab33_c2559198be04/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_e92da43d_cc4d_4246_bd30_24e4fea6a0e7/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_12dd8b6c_2268_424f_984a_2ca9aa81474d/datastream/OBJ/download) |    239M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_4dca445a_00da_400d_8ea2_c98c76d112e0/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_ebaa4ae6_7e68_4a9b_86a9_66f5c1870c9f/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_d9ebc33c_c3af_4e7d_8688_f7458d3b3c57/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_32db18c3_2407_44cf_bc5e_64b7099af04a/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_0fb6e644_3807_44d4_a6f4_fddcb4f70f88/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_87ac0d68_f46b_40fb_ad46_d4e8dd6831d5/datastream/OBJ/download) |     54M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_29225156_5ec7_4fbd_adf8_3041f1954ec2/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_941509cb_4726_4b9f_bd49_bdd3fe34cf4b/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_b75d5efd_b648_4a18_8782_b736782e4175/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_7d2b71ff_5260_42d9_9fdb_0c00cad5fd46/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_07e2b77a_1f0a_4ffc_a5b8_33e7f3c349de/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_f9569d1a_bae8_4dd5_9392_d38384257a16/datastream/OBJ/download) | | |
| Ukrainian | uk | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_4b31582d_c997_4d15_8aec_2fbf68df0426/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_43d8e748_457f_40b8_80d9_2e0ace36d79f/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_6c254171_e857_4550_a39d_62f8747d478b/datastream/OBJ/download) |   4945K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_e8e7b63f_c552_4260_a5e9_bb0d44d3d01d/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_dd442671_9938_4ac6_ab19_a0589a7eb19a/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_c1da1d23_3e5f_4bf0_b423_9a4da45f3719/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_aa841f46_207e_45cc_aae4_c47e10f6443f/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_3cd29cd0_b3b8_497e_aaec_f3112e3854ef/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_fbead7b8_d96a_4d46_892b_e32cd9d262a1/datastream/OBJ/download) |    162M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_933aa845_3e38_4c46_b986_aacfe434fea9/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_04801847_c1ad_4be0_8a1a_d09793923bd9/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_dcadb536_893f_4a2f_ad48_bccef6d401a7/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_8c4c10ee_c6a9_4788_8831_d7f921cb6001/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_d91fb042_dac5_4da1_99db_f98354432dca/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_0ef39396_c862_4320_ac6c_1212203b6b56/datastream/OBJ/download) | | |
| Urdu | ur | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_4165ec5e_27be_4e64_bb14_736b3297d40f/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_737b1259_02d7_4c3a_b322_1c41bc96705d/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_211c1462_ba18_4e7e_acfb_487af84bfbd4/datastream/OBJ/download) |    195K | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_e634fb11_f09b_41ab_95ba_167cdf347337/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_0419e169_28b1_46f0_8307_ec25567e928d/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_138b0ea5_b2e6_4bc8_baea_d2f13368308c/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_799e95be_e394_441a_bbfe_4c8c998b624a/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_33460c6a_65b3_43a5_a3a2_3fa8225692e2/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_a85de3b9_495c_4ce3_86b5_8468c841a0ad/datastream/OBJ/download) |     15M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_fc95997f_92e6_427a_b72b_60a4747c1614/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_be7691d7_2582_4bfd_a3d6_213930e6ff02/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_ed562097_1503_4cf2_b669_16a18179076e/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_06ce5ea2_8817_44e5_b1aa_2266aebbb134/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_1c390472_7730_401d_a328_b5fb8173b781/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_5947e601_20f4_49da_b307_930a54859745/datastream/OBJ/download) | | |
| Vietnamese | vi | OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_a5cc7725_1fc0_4dd2_8d3e_bce0a351db04/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_6d4a578d_0d5a_40e7_a7f7_ce4d8d3336ac/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_02adaf0e_be7b_46dd_b195_1cf4721e400d/datastream/OBJ/download) |     27M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_3fd48328_4113_4648_95ec_567df3bef818/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_0be6750c_9411_4772_affa_a0764130f540/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_ffaeba21_69e3_42e2_82e1_8cfcb909f7dd/datastream/OBJ/download) |
| | | Wikipedia | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_37473b5c_a324_49cb_9bd9_0fd78648b843/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_a9c813a2_d637_4306_b73a_43c75c728d18/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_6f0ece06_311f_4ce0_afc9_3d3bd78aa944/datastream/OBJ/download) |    115M | [word counts](https://archive.mpi.nl/islandora/object/lat%3A1839_6482e475_7d1f_4e2a_8de7_5affd860b402/datastream/OBJ/download)<br>[bigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_ba3b8caa_c8a6_4a9d_9b33_f61c82da33d1/datastream/OBJ/download)<br>[trigram counts](https://archive.mpi.nl/islandora/object/lat%3A1839_6f35d616_4e17_4d18_a415_3443d5edde98/datastream/OBJ/download) |
| | | Wikipedia + OpenSubtitles | [top 1M vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_09039ea6_2918_4f64_a315_225433748b1b/datastream/OBJ/download)<br>[all vectors](https://archive.mpi.nl/islandora/object/lat%3A1839_6349c0c1_eb27_4c45_a647_2f8d0287cae3/datastream/OBJ/download)<br>[model binary](https://archive.mpi.nl/islandora/object/lat%3A1839_14823a08_af81_4a33_b4ef_75640b492b69/datastream/OBJ/download) | | |
